#ifndef GTK_ACTION_GROUP_H
#define GTK_ACTION_GROUP_H

#include <gtk/gtk-action.h>

#define GTK_TYPE_ACTION_GROUP              (gtk_action_group_get_type ())
#define GTK_ACTION_GROUP(obj)              (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_ACTION_GROUP, GtkActionGroup))
#define GTK_ACTION_GROUP_CLASS(vtable)     (G_TYPE_CHECK_CLASS_CAST ((vtable), GTK_TYPE_ACTION_GROUP, GtkActionGroupClass))
#define GTK_IS_ACTION_GROUP(obj)           (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_ACTION_GROUP))
#define GTK_IS_ACTION_GROUP_CLASS(vtable)  (G_TYPE_CHECK_CLASS_TYPE ((vtable), GTK_TYPE_ACTION_GROUP))
#define GTK_ACTION_GROUP_GET_CLASS(inst)   (G_TYPE_INSTANCE_GET_CLASS ((inst), GTK_TYPE_ACTION_GROUP, GtkActionGroupClass))

typedef struct _GtkActionGroup      GtkActionGroup;
typedef struct _GtkActionGroupClass GtkActionGroupClass;
typedef struct _GtkActionGroupEntry GtkActionGroupEntry;

struct _GtkActionGroup
{
  GObject parent;

  gchar *name;
  GHashTable *actions;
};

struct _GtkActionGroupClass
{
  GObjectClass parent_class;

  GtkAction *(* get_action) (GtkActionGroup *action_group,
			     const gchar *action_name);
};

typedef enum {
  NORMAL_ACTION,
  TOGGLE_ACTION,
  RADIO_ACTION
} GtkActionGroupEntryType;

struct _GtkActionGroupEntry {
  gchar *name;
  gchar *label;
  gchar *stock_id;
  gchar *accelerator;
  gchar *tooltip;

  GCallback callback;
  gpointer user_data;

  GtkActionGroupEntryType entry_type;
  gchar *extra_data;
};

GType           gtk_action_group_get_type      (void);

GtkActionGroup *gtk_action_group_new           (const gchar *name);

const gchar    *gtk_action_group_get_name      (GtkActionGroup *action_group);
GtkAction      *gtk_action_group_get_action    (GtkActionGroup *action_group,
						const gchar *action_name);
GList          *gtk_action_group_list_actions  (GtkActionGroup *action_group);
void            gtk_action_group_add_action    (GtkActionGroup *action_group,
						GtkAction *action);
void            gtk_action_group_remove_action (GtkActionGroup *action_group,
						GtkAction *action);

void            gtk_action_group_add_actions   (GtkActionGroup *action_group,
						GtkActionGroupEntry *entries,
						guint n_entries);

#endif
