#ifndef GTK_ACTION_H
#define GTK_ACTION_H

#if 0
#include <gtk/gtkenums.h>
#include <gtk/gtktypeutils.h>
#include <gtk/gtkdebug.h>
#endif

#include <gtk/gtkwidget.h>
#include <glib-object.h>

#define GTK_TYPE_ACTION            (gtk_action_get_type ())
#define GTK_ACTION(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_ACTION, GtkAction))
#define GTK_ACTION_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_ACTION, GtkActionClass))
#define GTK_IS_ACTION(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_ACTION))
#define GTK_IS_ACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((obj), GTK_TYPE_ACTION))
#define GTK_ACTION_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj), GTK_TYPE_ACTION, GtkActionClass))

typedef struct _GtkAction      GtkAction;
typedef struct _GtkActionClass GtkActionClass;

struct _GtkAction
{
  GObject object;

  gchar *name;
  gchar *label;
  gchar *short_label;
  gchar *tooltip;
  gchar *stock_id; /* icon */

  guint sensitive : 1;
  guint visible : 1;
  guint label_set : 1;       /* these two used so we can set label */
  guint short_label_set : 1; /* based on stock id */

  /* accelerator */
  GQuark accel_quark;

  /* list of proxy widgets */
  GSList *proxies;
};

struct _GtkActionClass
{
  GObjectClass parent_class;

  /* activation signal */
  void       (* activate)           (GtkAction    *action);

  GType      menu_item_type;
  GType      toolbar_item_type;

  /* widget creation routines (not signals) */
  GtkWidget *(* create_menu_item)   (GtkAction *action);
  GtkWidget *(* create_tool_item)   (GtkAction *action);
  void       (* connect_proxy)      (GtkAction *action,
				     GtkWidget *proxy);
  void       (* disconnect_proxy)   (GtkAction *action,
				     GtkWidget *proxy);
};

GType gtk_action_get_type (void);

void       gtk_action_activate           (GtkAction *action);

GtkWidget *gtk_action_create_icon        (GtkAction *action,
					  GtkIconSize icon_size);
GtkWidget *gtk_action_create_menu_item   (GtkAction *action);
GtkWidget *gtk_action_create_tool_item   (GtkAction *action);
void       gtk_action_connect_proxy      (GtkAction *action,
					  GtkWidget *proxy);
void       gtk_action_disconnect_proxy   (GtkAction *action,
					  GtkWidget *proxy);

/* protected ... for use by child actions */
void gtk_action_block_activate_from   (GtkAction *action,
				       GtkWidget *proxy);
void gtk_action_unblock_activate_from (GtkAction *action,
				       GtkWidget *proxy);

/* protected ... for use by action groups */
void gtk_action_set_accel_path (GtkAction *action,
				const gchar *accel_path);


#endif
