#ifndef GTK_MENU_MERGE_H
#define GTK_MENU_MERGE_H

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtkaccelgroup.h>
#include <gtk/gtkwidget.h>
#include <gtk/gtk-action.h>
#include <gtk/gtk-action-group.h>

#define GTK_TYPE_MENU_MERGE            (gtk_menu_merge_get_type ())
#define GTK_MENU_MERGE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_MENU_MERGE, GtkMenuMerge))
#define GTK_MENU_MERGE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_MENU_MERGE, GtkMenuMergeClass))
#define GTK_IS_MENU_MERGE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_MENU_MERGE))
#define GTK_IS_MENU_MERGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((obj), GTK_TYPE_MENU_MERGE))
#define GTK_MENU_MERGE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj), GTK_TYPE_MENU_MERGE, GtkMenuMergeClass))

typedef struct _GtkMenuMerge      GtkMenuMerge;
typedef struct _GtkMenuMergeClass GtkMenuMergeClass;
typedef struct _GtkMenuMergeNode  GtkMenuMergeNode;

typedef enum {
  GTK_MENU_MERGE_UNDECIDED,
  GTK_MENU_MERGE_ROOT,
  GTK_MENU_MERGE_MENUBAR,
  GTK_MENU_MERGE_MENU,
  GTK_MENU_MERGE_TOOLBAR,
  GTK_MENU_MERGE_MENU_PLACEHOLDER,
  GTK_MENU_MERGE_TOOLBAR_PLACEHOLDER,
  GTK_MENU_MERGE_POPUPS,
  GTK_MENU_MERGE_MENUITEM,
  GTK_MENU_MERGE_TOOLITEM,
  GTK_MENU_MERGE_SEPARATOR,
} GtkMenuMergeNodeType;

struct _GtkMenuMerge {
  GObject parent;

  GtkAccelGroup *accel_group;

  GNode *root_node;
  GList *action_groups;

  guint last_merge_id;

  guint update_tag;
};

struct _GtkMenuMergeClass {
  GObjectClass parent_class;

  void (* add_widget) (GtkMenuMerge *merge, GtkWidget *widget);
  void (* remove_widget) (GtkMenuMerge *merge, GtkWidget *widget);
};

struct _GtkMenuMergeNode {
  GtkMenuMergeNodeType type;

  const gchar *name;

  GQuark action_name;
  GtkAction *action;
  GtkWidget *proxy;
  GtkWidget *extra; /*GtkMenu for submenus, second separator for placeholders*/

  GList *uifiles;

  guint dirty : 1;
};

GType         gtk_menu_merge_get_type            (void);
GtkMenuMerge *gtk_menu_merge_new                 (void);

/* these two functions will dirty all merge nodes, as they may need to
 * be connected up to different actions */
void          gtk_menu_merge_insert_action_group (GtkMenuMerge *self,
						  GtkActionGroup *action_group,
						  gint pos);
void          gtk_menu_merge_remove_action_group (GtkMenuMerge *self,
						  GtkActionGroup*action_group);


GtkWidget    *gtk_menu_merge_get_widget          (GtkMenuMerge *self,
						  const gchar *path);

/* these two functions are for adding UI elements to the merged user
 * interface */
guint         gtk_menu_merge_add_ui_from_string  (GtkMenuMerge *self,
						  const gchar *buffer,
						  guint length,
						  GError **error);
guint         gtk_menu_merge_add_ui_from_file    (GtkMenuMerge *self,
						  const gchar *filename,
						  GError **error);
void          gtk_menu_merge_remove_ui           (GtkMenuMerge *self,
						  guint merge_id);

void          gtk_menu_merge_ensure_update       (GtkMenuMerge *self);


#endif /* GTK_MENU_MERGE_H */
