#include "gtk-radio-action.h"

static void gtk_radio_action_init       (GtkRadioAction *action);
static void gtk_radio_action_class_init (GtkRadioActionClass *class);

GType
gtk_radio_action_get_type (void)
{
  static GtkType type = 0;

  if (!type)
    {
      static const GTypeInfo type_info =
      {
        sizeof (GtkRadioActionClass),
        (GBaseInitFunc) NULL,
        (GBaseFinalizeFunc) NULL,
        (GClassInitFunc) gtk_radio_action_class_init,
        (GClassFinalizeFunc) NULL,
        NULL,
        
        sizeof (GtkRadioAction),
        0, /* n_preallocs */
        (GInstanceInitFunc) gtk_radio_action_init,
      };

      type = g_type_register_static (GTK_TYPE_TOGGLE_ACTION,
                                     "GtkRadioAction",
                                     &type_info, 0);
    }
  return type;
}

static void gtk_radio_action_finalize (GObject *object);
static void gtk_radio_action_activate (GtkAction *action);

static GObjectClass *parent_class = NULL;

static void
gtk_radio_action_class_init (GtkRadioActionClass *class)
{
  GObjectClass *object_class;
  GtkActionClass *action_class;

  parent_class = g_type_class_peek_parent (class);
  object_class = G_OBJECT_CLASS (class);
  action_class = GTK_ACTION_CLASS (class);

  object_class->finalize = gtk_radio_action_finalize;

  action_class->activate = gtk_radio_action_activate;
}

static void
gtk_radio_action_init (GtkRadioAction *action)
{
  action->group = g_slist_prepend (NULL, action);
}

static void
gtk_radio_action_finalize (GObject *object)
{
  GtkRadioAction *action;
  GSList *tmp_list;

  g_return_if_fail (GTK_IS_RADIO_ACTION (object));

  action = GTK_RADIO_ACTION (object);

  action->group = g_slist_remove (action->group, action);

  tmp_list = action->group;

  while (tmp_list)
    {
      GtkRadioAction *tmp_action = tmp_list->data;

      tmp_list = tmp_list->next;
      tmp_action->group = action->group;
    }

  if (parent_class->finalize)
    (* parent_class->finalize) (object);
}

static void
gtk_radio_action_activate (GtkAction *action)
{
  GtkRadioAction *radio_action;
  GtkToggleAction *toggle_action;
  GtkToggleAction *tmp_action;
  GSList *tmp_list;

  g_return_if_fail (GTK_IS_RADIO_ACTION (action));

  radio_action = GTK_RADIO_ACTION (action);
  toggle_action = GTK_TOGGLE_ACTION (action);

  if (toggle_action->active)
    {
      tmp_list = radio_action->group;

      while (tmp_list)
	{
	  tmp_action = tmp_list->data;
	  tmp_list = tmp_list->next;

	  if (tmp_action->active && (tmp_action != toggle_action)) 
	    {
	      toggle_action->active = !toggle_action->active;
	      break;
	    }
	}
    }
  else
    {
      toggle_action->active = !toggle_action->active;

      tmp_list = radio_action->group;
      while (tmp_list)
	{
	  tmp_action = tmp_list->data;
	  tmp_list = tmp_list->next;

	  if (tmp_action->active && (tmp_action != toggle_action))
	    {
	      gtk_action_activate (GTK_ACTION (tmp_action));
	      break;
	    }
	}
    }

  gtk_toggle_action_toggled (toggle_action);
}

/**
 * gtk_radio_action_get_group:
 * @action: the action object
 *
 * Returns: the list representing the radio group for this object
 */
GSList *
gtk_radio_action_get_group (GtkRadioAction *action)
{
  g_return_val_if_fail (GTK_IS_RADIO_ACTION (action), NULL);

  return action->group;
}

/**
 * gtk_radio_action_set_group:
 * @action: the action object
 * @group: a list representing a radio group
 *
 * Sets the radio group for the radio action object.
 */
void
gtk_radio_action_set_group (GtkRadioAction *action, GSList *group)
{
  g_return_if_fail (GTK_IS_RADIO_ACTION (action));
  g_return_if_fail (!g_slist_find (group, action));

  if (action->group)
    {
      GSList *slist;

      action->group = g_slist_remove (action->group, action);

      for (slist = action->group; slist; slist = slist->next)
	{
	  GtkRadioAction *tmp_action = slist->data;

	  tmp_action->group = action->group;
	}
    }

  action->group = g_slist_prepend (group, action);

  if (group)
    {
      GSList *slist;

      for (slist = action->group; slist; slist = slist->next)
	{
	  GtkRadioAction *tmp_action = slist->data;

	  tmp_action->group = action->group;
	}
    }
  else
    {
      GTK_TOGGLE_ACTION (action)->active = TRUE;
    }
}
